! This collection of routines contains code that is not routinely
! used in the model. Subroutines should be extracted as needed
! and compiled and linked into the model. It contains, for example,
! routines for interporlating a data field to higher resolution. 
! Some routines have the same names as those found elsewhere.
!
! It is not documented except in the code itself
!
!-----------------------------------------------------------------------
!  The following two routines (pgin, convert) plus pgintv allow
!   interpolation to higher resolution.  The resolution is
!   doubled in x and y, while the resolution in z may be either
!   doubled or held fixed.
!  To use, replace the single routine `pgin' in pgio_cdf with
!   the collection of the two following routines plus pgintv.
!
      subroutine pgin(t,s,r,p0)
! This routine is for reading lower horizontal resolution arrays only.
! It reads the lower dimension arrays, that are locally defined, by
! calling pgintv(). Then routine convert is called, which interpolates
! to a higher resolution. 
!
! This version is for pgom, i.e., assumes an explicit (stretched)
! vertical grid.  It only works for doubling resolution in horizontal
! and vertical.  But the structure is such that this could be
! changed easily enough.
!
      include 'params.inc'
      include 'common.inc'
!  Smaller (input) array size parameters:
!   For doubled vertical resolution:
      parameter (NXO=(NX+1)/2,NYO=(NY+1)/2,NZO=NZ/2)
!   For constant vertical resolution:
!      parameter (NXO=(NX+1)/2,NYO=(NY+1)/2,NZO=NZ)
      include '/usr/local/include/netcdf.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),p0(NX,NY)
	 real tol(NXO,NYO,0:NZO+1),sol(NXO,NYO,0:NZO+1),
     +    rol(NXO,NYO,0:NZO+1),p0ol(NXO,NYO)
      real*4 x(NXO)  ! these next few lines needed for pgintv
      real*4 y(NYO)
      real*4 zol(0:NZO+1)
      real*4 v0(NXO,NYO,NZO+2)
!
! the first task is to read in the old (low resolution) data. This is
! done by a call to pgint. This is almost identical to pgin, except 
! that the dimensioning values and various arrays are passed in the 
! calling statement. 
!
	call pgintv(tol,sol,rol,p0ol,x,y,zol,v0,isal,ifile,NXO,NYO,NZO)
!
! The next thing is to convert these to higher resolution. This is
! done by linear interpolation. For modularity (ugh!) we do this by a
! call to another routine, convert. (Actually, this has an advantage 
! in that if it were desired to have a more general resolution-changer,
! this would be the only routine that needed major changes.)
!
	call convert(t,s,r,p0,tol,sol,rol,p0ol,z,zol
     .,              NX,NY,NZ,NXO,NYO,NZO)
!
! all done!
	return
	end

!
!------------
!

      subroutine convert(t,s,r,p0,tol,sol,rol,p0ol,z,zol
     .,                  NX,NY,NZ,NXO,NYO,NZO)
! This is a relatively straightforward routine that interpolates to
! higher resolution. 
! In: tol,sol,rol,p0ol
! Out: t,s,r,p0
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),p0(NX,NY)
      real tol(NXO,NYO,0:NZO+1),sol(NXO,NYO,0:NZO+1),
     +    rol(NXO,NYO,0:NZO+1),p0ol(NXO,NYO)
      real z(0:NZ+1)
      real*4 zol(0:NZO+1)
c
!  For stretched grid, have to change the logic a bit:
!   First do vertical interpolation at each common (odd,odd) x-y point.
!   Then do the horizontal interpolation using the new field, instead
!    of the old as before
!   Start by doing the first step on the Ekman layer and the bottom
      do j=1,NY,2    
       do i=1,NX,2 	
        t(i,j,NZ+1)=tol((i+1)/2,(j+1)/2,NZO+1)
        s(i,j,NZ+1)=sol((i+1)/2,(j+1)/2,NZO+1)
        r(i,j,NZ+1)=rol((i+1)/2,(j+1)/2,NZO+1)
        t(i,j,0)=tol((i+1)/2,(j+1)/2,0)
        s(i,j,0)=sol((i+1)/2,(j+1)/2,0)
        r(i,j,0)=rol((i+1)/2,(j+1)/2,0)
       enddo
      enddo
!   Do the vertical interpolation
      kol=0
      do k=1,NZ
 10    continue
!  We assume that z(k).gt.zol(kol) always, and it should be
       if(z(k).lt.zol(kol+1)) goto 20
        kol=kol+1
        if(kol.gt.NZ+1) stop ' convert: kol'
        write(6,'('' k,kol= '',2i6,2e12.4)') k,kol,z(k),zol(kol+1)
       goto 10
 20    continue
       zfac=(z(k)-zol(kol))/(zol(kol+1)-zol(kol))
       do j=1,NY,2    
        do i=1,NX,2 	
         t(i,j,k)=(1.-zfac)*tol((i+1)/2,(j+1)/2,kol)
     .                +zfac*tol((i+1)/2,(j+1)/2,kol+1)
         s(i,j,k)=(1.-zfac)*sol((i+1)/2,(j+1)/2,kol)
     .                +zfac*sol((i+1)/2,(j+1)/2,kol+1)
         r(i,j,k)=(1.-zfac)*rol((i+1)/2,(j+1)/2,kol)
     .                +zfac*rol((i+1)/2,(j+1)/2,kol+1)
        enddo
       enddo
      enddo
!  Do the horizontal interpolation
         do k=0,NZ+1
	  do j=1,NY,2	  
	    do i=2,NX-1,2	
	     t(i,j,k)=0.5*(t(i-1,j,k)+t(i+1,j,k))
	     s(i,j,k)=0.5*(s(i-1,j,k)+s(i+1,j,k))
	     r(i,j,k)=0.5*(r(i-1,j,k)+r(i+1,j,k))
c	     t(i,j,k)=0.5*(tol(i/2,(j+1)/2,k)+tol(i/2+1,(j+1)/2,k))
	    enddo
	  enddo
	  do j=2,NY-1,2	 
	    do i=1,NX,2	   
	     t(i,j,k)=0.5*(t(i,j-1,k)+t(i,j+1,k))
	     s(i,j,k)=0.5*(s(i,j-1,k)+s(i,j+1,k))
	     r(i,j,k)=0.5*(r(i,j-1,k)+r(i,j+1,k))
! 	     t(i,j,k)=0.5*(tol((i+1)/2,j/2,k)+tol((i+1)/2,j/2+1,k))
	    enddo
	  enddo
	  do j=2,NY-1,2	 
	    do i=2,NX-1,2    
	     t(i,j,k)=0.25*(t(i-1,j-1,k)+t(i-1,j+1,k)
     +	                 +  t(i+1,j-1,k)+t(i+1,j+1,k))
	     s(i,j,k)=0.25*(s(i-1,j-1,k)+s(i-1,j+1,k)
     +	                 +  s(i+1,j-1,k)+s(i+1,j+1,k))
	     r(i,j,k)=0.25*(r(i-1,j-1,k)+r(i-1,j+1,k)
     +	                 +  r(i+1,j-1,k)+r(i+1,j+1,k))
c	     t(i,j,k)=0.25*(tol(i/2,j/2,k)+tol(i/2,j/2+1,k)
!     +	                 + tol(i/2+1,j/2,k)+tol(i/2+1,j/2+1,k))
	    enddo
	  enddo
	enddo
! Now  pressure (used for first guess)
	  do j=1,NY,2
	    do i=1,NX,2
	     p0(i,j)=p0ol((i+1)/2,(j+1)/2)
	    enddo
	  enddo
	  do j=1,NY,2
	    do i=2,NX-1,2
	     p0(i,j)=0.5*(p0ol(i/2,(j+1)/2)+p0ol(i/2+1,(j+1)/2))
	    enddo
	  enddo
	  do j=2,NY-1,2
	    do i=1,NX,2
	     p0(i,j)=0.5*(p0ol((i+1)/2,j/2)+p0ol((i+1)/2,j/2+1))
	   enddo
	  enddo
	  do j=2,NY-1,2
	    do i=2,NX-1,2
	     p0(i,j)=0.25*(p0ol(i/2,j/2)+p0ol(i/2,j/2+1)
     +	                 + p0ol(i/2+1,j/2)+p0ol(i/2+1,j/2+1))
	    enddo
	  enddo
	return
	end

!
!-----------------------------------------------------------------------
!  The following two routines (pgin, converz) plus pgintv allow
!   interpolation to a different vertical grid with fixed
!   horizontal resolution.
!  To use, replace the single routine `pgin' in pgio_cdf with
!   the collection of the two following routines plus pgintv.
!
      subroutine pgin(t,s,r,p0)
! This routine is for reading lower vertical resolution arrays only.
! It reads the lower dimension arrays, that are locally defined, by
! calling pgintv(). Then routine converz is called, which interpolates
! to a different vertical grid. 
!
! This version is for pgom, i.e., assumes an explicit (stretched)
! vertical grid.  It only works for changing resolution in vertical.
!
      include 'params.inc'
      include 'common.inc'
!  Smaller (input) array size parameters:
!   Fixed number of grid levels, different grid
      parameter (NZO=NZ)
!   Doubled number of grid levels
!      parameter (NZO=NZ/2)
      include '/usr/local/include/netcdf.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),p0(NX,NY)
      real tol(NX,NY,0:NZO+1),sol(NX,NY,0:NZO+1),rol(NX,NY,0:NZO+1)
      real*4 x(NX)  ! these next few lines needed for pgintv
      real*4 y(NY)
      real*4 zol(0:NZO+1)
      real*4 v0(NX,NY,NZO+2)
!
! the first task is to read in the old (low resolution) data. This is
! done by a call to pgint. This is almost identical to pgin, except 
! that the dimensioning values and various arrays are passed in the 
! calling statement. 
!
	call pgintv(tol,sol,rol,p0,x,y,zol,v0,isal,ifile,NX,NY,NZO)
!
! The next thing is to convert these to higher resolution. This is
! done by linear interpolation. For modularity (ugh!) we do this by a
! call to another routine, convert. (Actually, this has an advantage 
! in that if it were desired to have a more general resolution-changer,
! this would be the only routine that needed major changes.)
!
	call converz(t,s,r,tol,sol,rol,z,zol,NX,NY,NZ,NZO)
!
! all done!
	return
	end
!
!-----------
!

      subroutine converz(t,s,r,tol,sol,rol,z,zol,NX,NY,NZ,NZO)
! This is a relatively straightforward routine that interpolates to
!  higher vertical resolution. 
! In: tol,sol,rol
! Out: t,s,r
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1)
      real tol(NX,NY,0:NZO+1),sol(NX,NY,0:NZO+1),rol(NX,NY,0:NZO+1)
      real z(0:NZ+1)
      real*4 zol(0:NZO+1)
!
!  For stretched grid, have to change the logic a bit:
!   First do vertical interpolation at each common (odd,odd) x-y point.
!   Then do the horizontal interpolation using the new field, instead
!    of the old as before
!   Start by doing the first step on the Ekman layer and the bottom
      do j=1,NY
       do i=1,NX 	
        t(i,j,NZ+1)=tol(i,j,NZO+1)
        s(i,j,NZ+1)=sol(i,j,NZO+1)
        r(i,j,NZ+1)=rol(i,j,NZO+1)
        t(i,j,0)=tol(i,j,0)
        s(i,j,0)=sol(i,j,0)
        r(i,j,0)=rol(i,j,0)
       enddo
      enddo
!   Do the vertical interpolation
      kol=0
      do k=1,NZ
 10    continue
!  We assume that z(k).gt.zol(kol) always, and it should be
       if(z(k).lt.zol(kol+1)) goto 20
        kol=kol+1
        if(kol.gt.NZ+1) stop ' converz: kol'
        write(6,'('' k,kol= '',2i6,2e12.4)') k,kol,z(k),zol(kol+1)
       goto 10
 20    continue
       zfac=(z(k)-zol(kol))/(zol(kol+1)-zol(kol))
       do j=1,NY  
        do i=1,NX 	
         t(i,j,k)=(1.-zfac)*tol(i,j,kol)+zfac*tol(i,j,kol+1)
         s(i,j,k)=(1.-zfac)*sol(i,j,kol)+zfac*sol(i,j,kol+1)
         r(i,j,k)=(1.-zfac)*rol(i,j,kol)+zfac*rol(i,j,kol+1)
        enddo
       enddo
      enddo
      return
      end
!
!-----------------------------------------------------------------------
!   This routine is for use with the attached interpolation routines
!    (pgin + convert, pgin + converz)
!
      subroutine pgintv(t,s,r,p0,x,y,z1,v0,isal,ifile,NX,NY,NZ)
!  Read thermocline array r, p0 from NetCDF (EPIC) format for restart

! Note that we do not include the parameter statement as its values
! for NX and NY are now different. This in turn means we can't have the
! common block in here, as these contain arrays dimensioned via the
! parameter statements.  Thus we must pass various arrays. 
!
      include '/usr/local/include/netcdf.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),p0(NX,NY)
      integer  iret
      character*6 ifile  ! need this as common block is now absent
! netCDF id
      integer  ncid
! dimension ids
      integer  idim,jdim,kdim,ldim
! variable ids
      integer  xid,yid,zid,timeid,v1id,v2id,v6id,v7id
! starts and counts for slabs
      integer start(1), count(1),start4(4),count4(4)
! index for ncvgt1
      integer mindex1(1)
      integer mindex4(4)
! data variables
      real*4 x(NX)
      real*4 y(NY)
      real*4 z1(NZ+2)
      real*4 value
      real*4 v0(NX,NY,NZ+2)
!  The following assumes that rt is real*8, otherwise we'll be out
!   of bounds!
!      equivalence (v0(1,1,0),rt(1,1,1))
      integer time
! Dummy dimension name
      character*31 dimnam
!
!  Open NetCDF input file
      ncid = ncopn (ifile//'.cdf', NCNOWRIT, iret)
!  Determine dimension id's
      xid = ncdid (ncid, 'x', iret)
      yid = ncdid (ncid, 'y', iret)
      zid = ncdid (ncid, 'z', iret)
      timeid = ncdid (ncid, 'time', iret)
!  Check dimension values
      call ncdinq (ncid,xid,dimnam,idim,iret)
      call ncdinq (ncid,yid,dimnam,jdim,iret)
      call ncdinq (ncid,zid,dimnam,kdim,iret)
      call ncdinq (ncid,timeid,dimnam,ldim,iret)
      if(idim.ne.NX .or. jdim.ne.NY .or. kdim.ne.NZ+2 .or. ldim.ne.1)
     .  stop
!  Determine variable id's
      v1id = ncvid (ncid, 'rho', iret)
      v2id = ncvid (ncid, 'p', iret)
      if(isal.eq.1) then
       v6id = ncvid (ncid, 't', iret)
       v7id = ncvid (ncid, 's', iret)
      endif
!  Get the vertical grid z(k+1)
      start(1) = 1
      count(1) = NZ+2
      call ncvgt(ncid, zid, start, count, z1, iret)
!  Get the entire slab in v0, then store
      start4(1)=1
      start4(2)=1
      start4(3)=1
      start4(4)=1
      count4(1)=NX
      count4(2)=NY
      count4(3)=NZ+2
      count4(4)=1
!  Get rho
      call ncvgt(ncid, v1id, start4, count4, v0, iret)
      do k=0,NZ+1
       do j=1,NY
        do i=1,NX
         r(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!  Get p0 from p(i,j,k=NZ+2)
      call ncvgt(ncid, v2id, start4, count4, v0, iret)
      do j=1,NY
       do i=1,NX
        p0(i,j)=dble(v0(i,j,NZ+2))
       enddo
      enddo
!  Temperature and salinity
      if(isal.eq.1) then
       call ncvgt(ncid, v6id, start4, count4, v0, iret)
       do k=0,NZ+1
        do j=1,NY
         do i=1,NX
          t(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
       call ncvgt(ncid, v7id, start4, count4, v0, iret)
       do k=0,NZ+1
        do j=1,NY
         do i=1,NX
          s(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
      else
       do k=0,NZ+1
        do j=1,NY
         do i=1,NX
          t(i,j,k)=-r(i,j,k)
         enddo
        enddo
       enddo
      endif
!  Close ncdf file
      call ncclos(ncid,iret)
      return
      end

!
!-----------------------------------------------------------------------
!
!
      subroutine pgin1(t,s,r,p0,p,u,v,w)
!  Read all arrays from NetCDF (EPIC) format output file
      include 'params.inc'
      include 'common.inc'
      include '/usr/local/include/netcdf.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1)
      real p0(NX,NY),p(NX,NY,0:NZ+1)
      real u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)
      integer  iret
! netCDF id
      integer  ncid
! dimension ids
      integer  idim,jdim,kdim,ldim
! variable ids
      integer  xid,yid,zid,timeid,v1id,v2id,v3id,v4id,v5id,v6id,v7id
! starts and counts for slabs
      integer start(1), count(1),start4(4),count4(4)
c index for ncvgt1
      integer mindex1(1)
      integer mindex4(4)
! data variables
      real*4 x(NX)
      real*4 y(NY)
      real*4 z1(NZ+2)
      real*4 value
      real*4 v0(NX,NY,NZ+2)
      integer time
! Dummy dimension name
      character*31 dimnam
!  Open NetCDF input file
      ncid = ncopn (ifile//'.cdf', NCNOWRIT, iret)
!  Determine dimension id's
      xid = ncdid (ncid, 'x', iret)
      yid = ncdid (ncid, 'y', iret)
      zid = ncdid (ncid, 'z', iret)
      timeid = ncdid (ncid, 'time', iret)
!  Check dimension values
      call ncdinq (ncid,xid,dimnam,idim,iret)
      call ncdinq (ncid,yid,dimnam,jdim,iret)
      call ncdinq (ncid,zid,dimnam,kdim,iret)
      call ncdinq (ncid,timeid,dimnam,ldim,iret)
      if(idim.ne.NX .or. jdim.ne.NY .or. kdim.ne.NZ+2 .or. ldim.ne.1)
     .  stop
!  Determine variable id's
      v1id = ncvid (ncid, 'rho', iret)
      v2id = ncvid (ncid, 'p', iret)
      v3id = ncvid (ncid, 'u', iret)
      v4id = ncvid (ncid, 'v', iret)
      v5id = ncvid (ncid, 'w', iret)
      if(isal.eq.1) then
       v6id = ncvid (ncid, 't', iret)
       v7id = ncvid (ncid, 's', iret)
      endif
!  Get dimension arrays
!  Get x
      start(1) = 1
      count(1) = NX
      call ncvgt(ncid, xid, start, count, x, iret)
!  Get y
      start(1) = 1
      count(1) = NY
      call ncvgt(ncid, yid, start, count, y, iret)
!  Get z
      start(1) = 1
      count(1) = NZ+2
      call ncvgt(ncid, zid, start, count, z1, iret)
!  Following sets vertical grid array in common from .cdf file:
!      do k=0,NZ+1
!       z(k)=z1(k+1)
!      enddo
!  Get the entire slab in v0, then store
      start4(1)=1
      start4(2)=1
      start4(3)=1
      start4(4)=1
      count4(1)=NX
      count4(2)=NY
      count4(3)=NZ+2
      count4(4)=1
!  Get rho
      call ncvgt(ncid, v1id, start4, count4, v0, iret)
      do k=1,NZ+1
       do j=1,NY
        do i=1,NX
         r(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!   p (note p(k=NZ+1)=p0, barotropic pressure)
      call ncvgt(ncid, v2id, start4, count4, v0, iret)
      do k=1,NZ+1
       do j=1,NY
        do i=1,NX
         p(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!  Get p0 from p(i,j,k=NZ+2)
      do j=1,NY
       do i=1,NX
        p0(i,j)=p(i,j,NZ+1)
       enddo
      enddo
!   u
      call ncvgt(ncid, v3id, start4, count4, v0, iret)
      do k=1,NZ+1
       do j=1,NY
        do i=1,NX
         u(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!   v
      call ncvgt(ncid, v4id, start4, count4, v0, iret)
      do k=1,NZ+1
       do j=1,NY
        do i=1,NX
         v(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!   w
      call ncvgt(ncid, v5id, start4, count4, v0, iret)
      do k=0,NZ+1
       do j=1,NY
        do i=1,NX
         w(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!  Temperature and salinity
      if(isal.eq.1) then
       call ncvgt(ncid, v6id, start4, count4, v0, iret)
       do k=1,NZ+1
        do j=1,NY
         do i=1,NX
          t(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
       call ncvgt(ncid, v7id, start4, count4, v0, iret)
       do k=1,NZ+1
        do j=1,NY
         do i=1,NX
          s(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
      else
       do k=1,NZ+1
        do j=1,NY
         do i=1,NX
          t(i,j,k)=-r(i,j,k)
         enddo
        enddo
       enddo
      endif
!  Close ncdf file
      call ncclos(ncid,iret)
      return
      end
!
! ------------------------------------------------------------------
!
      subroutine pgin(t,s,r,p0)
!  Read thermocline array r, p0 from NetCDF (EPIC) format for restart
! This subroutine can be used with machines (such as IBM RS6000s) 
! on which the double precision flag over-rides the source code. 
! This routine should be compiled *without* any double-precision
! flag, before being linked in with the rest of the model.

       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'params.inc'
      include 'common.inc'
      include '/usr/local/include/netcdf.inc'
      dimension t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),
     +   p0(NX,NY)
      integer  iret
! netCDF id
      integer  ncid
! dimension ids
      integer  idim,jdim,kdim,ldim
! variable ids
      integer  xid,yid,zid,timeid,v1id,v2id,v6id,v7id
! starts and counts for slabs
      integer start(1), count(1),start4(4),count4(4)
! index for ncvgt1
      integer mindex1(1)
      integer mindex4(4)
! data variables
      real*4 x(NX)
      real*4 y(NY)
      real*4 z(NZ+2)
      real*4 value
      real*4 v0(NX,NY,NZ+2)
      integer time
! Dummy dimension name
      character*31 dimnam
!
!  Open NetCDF input file
      ncid = ncopn (ifile//'.cdf', NCNOWRIT, iret)
!  Determine dimension id's
      xid = ncdid (ncid, 'x', iret)
      yid = ncdid (ncid, 'y', iret)
      zid = ncdid (ncid, 'z', iret)
      timeid = ncdid (ncid, 'time', iret)
!  Check dimension values
      call ncdinq (ncid,xid,dimnam,idim,iret)
      call ncdinq (ncid,yid,dimnam,jdim,iret)
      call ncdinq (ncid,zid,dimnam,kdim,iret)
      call ncdinq (ncid,timeid,dimnam,ldim,iret)
      if(idim.ne.NX .or. jdim.ne.NY .or. kdim.ne.NZ+2 .or. ldim.ne.1)
     .  stop
!  Determine variable id's
      v1id = ncvid (ncid, 'rho', iret)
      v2id = ncvid (ncid, 'p', iret)
      if(isal.eq.1) then
       v6id = ncvid (ncid, 't', iret)
       v7id = ncvid (ncid, 's', iret)
      endif
!  Get the entire slab in v0, then store
      start4(1)=1
      start4(2)=1
      start4(3)=1
      start4(4)=1
      count4(1)=NX
      count4(2)=NY
      count4(3)=NZ+2
      count4(4)=1
!  Get rho
      call ncvgt(ncid, v1id, start4, count4, v0, iret)
      do k=1,NZ+1
       do j=2,NY-1
        do i=2,NX-1
         r(i,j,k)=dble(v0(i,j,k+1))
        enddo
       enddo
      enddo
!  Get p0 from p(i,j,k=NZ+2)
      call ncvgt(ncid, v2id, start4, count4, v0, iret)
      do j=1,NY
       do i=1,NX
        p0(i,j)=dble(v0(i,j,NZ+2))
       enddo
      enddo
!  Temperature and salinity
      if(isal.eq.1) then
       call ncvgt(ncid, v6id, start4, count4, v0, iret)
       do k=1,NZ+1
        do j=2,NY-1
         do i=2,NX-1
          t(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
       call ncvgt(ncid, v7id, start4, count4, v0, iret)
       do k=1,NZ+1
        do j=2,NY-1
         do i=2,NX-1
          s(i,j,k)=dble(v0(i,j,k+1))
         enddo
        enddo
       enddo
      else
       do k=1,NZ+1
        do j=2,NY-1
         do i=2,NX-1
          t(i,j,k)=-r(i,j,k)
         enddo
        enddo
       enddo
      endif
!  Close ncdf file
      call ncclos(ncid,iret)
      return
      end
!
!-----------------------------------------------------------------------
!
      subroutine pgout(itc,time,t,s,r,p0,p,u,v,w)
!  Write thermocline arrays in NetCDF (EPIC) format
! This subroutine can be used with machines (such as IBM RS6000s) 
! on which the double precision flag over-rides the source code. 
! This routine should be compiled *without! any double-precision
! flag, before being linked in with the rest of the model.

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      include 'params.inc'
      include 'common.inc'
      include '/usr/local/include/netcdf.inc'
      dimension t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1)
      dimension r(NX,NY,0:NZ+1),p0(NX,NY),p(NX,NY,NZ+1)
      dimension u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)
      integer  iret
! netCDF id
      integer  ncid
! dimension ids
      integer  idim,jdim,kdim,ldim
! variable ids
      integer  xid,yid,zid,timeid,v1id,v2id,v3id,v4id,v5id,v6id,v7id
! variable shapes
      integer dims(4)
! starts and counts
      integer start(1), count(1),start4(4),count4(4)
! index for ncvpt1
      integer mindex1(1)
      integer mindex4(4)
! data variables
      real*4 x(NX)
      real*4 y(NY)
      real*4 z(NZ+2)
      real*4 value
      real*4 v0(NX*NY*(NZ+2)*1)
      integer time
! attribute vectors
      integer  longval(1)
!
! enter define mode
      ncid = nccre (ofile//'.cdf', NCCLOB, iret)
! define dimensions
      idim = ncddef(ncid, 'x', NX, iret)
      jdim = ncddef(ncid, 'y', NY, iret)
      kdim = ncddef(ncid, 'z', NZ+2, iret)
      ldim = ncddef(ncid, 'time', 1, iret)
! define variables
      dims(1) = idim
      xid = ncvdef (ncid, 'x', NCFLOAT, 1, dims, iret)
      dims(1) = jdim
      yid = ncvdef (ncid, 'y', NCFLOAT, 1, dims, iret)
      dims(1) = kdim
      zid = ncvdef (ncid, 'z', NCFLOAT, 1, dims, iret)
      dims(1) = ldim
      timeid = ncvdef (ncid, 'time', NCLONG, 1, dims, iret)
      dims(4) = ldim
      dims(3) = kdim
      dims(2) = jdim
      dims(1) = idim
      v1id = ncvdef (ncid, 'rho', NCFLOAT, 4, dims, iret)
      v2id = ncvdef (ncid, 'p', NCFLOAT, 4, dims, iret)
      v3id = ncvdef (ncid, 'u', NCFLOAT, 4, dims, iret)
      v4id = ncvdef (ncid, 'v', NCFLOAT, 4, dims, iret)
      v5id = ncvdef (ncid, 'w', NCFLOAT, 4, dims, iret)
      if(isal.eq.1) then
       v6id = ncvdef (ncid, 't', NCFLOAT, 4, dims, iret)
       v7id = ncvdef (ncid, 's', NCFLOAT, 4, dims, iret)
      endif
! assign attributes
      call ncaptc(ncid, xid, 'units', NCCHAR, 2, 'L0', iret)
      call ncaptc(ncid, yid, 'units', NCCHAR, 2, 'L0', iret)
      call ncaptc(ncid, zid, 'units', NCCHAR, 1, 'D', iret)
      call ncaptc(ncid, timeid, 'units', NCCHAR, 1, 'T', iret)
      longval(1) = 501
      call ncapt(ncid, xid, 'epic_code', NCLONG, 1, longval, iret)
      longval(1) = 500
      call ncapt(ncid, yid, 'epic_code', NCLONG, 1, longval, iret)
      longval(1) = 1
      call ncapt(ncid, zid, 'epic_code', NCLONG, 1, longval, iret)
      longval(1) = 625
      call ncapt(ncid, timeid, 'epic_code', NCLONG, 1, longval, iret)
! assign variable attributes
      call ncaptc(ncid, v1id, 'units', NCCHAR, 4, 'Rho0', iret)
      call ncaptc(ncid, v1id, 'long_name', NCCHAR, 7, 'Density', iret)
      call ncaptc(ncid, v2id, 'units', NCCHAR, 5, 'Rho0D', iret)
      call ncaptc(ncid, v2id, 'long_name', NCCHAR, 8, 'Pressure', iret)
      call ncaptc(ncid, v3id, 'units', NCCHAR, 1, 'U', iret)
      call ncaptc(ncid, v3id, 'long_name', NCCHAR, 1, 'u', iret)
      call ncaptc(ncid, v4id, 'units', NCCHAR, 1, 'U', iret)
      call ncaptc(ncid, v4id, 'long_name', NCCHAR, 1, 'v', iret)
      call ncaptc(ncid, v5id, 'units', NCCHAR, 4, 'DU/L', iret)
      call ncaptc(ncid, v5id, 'long_name', NCCHAR, 1, 'w', iret)
      if(isal.eq.1) then
       call ncaptc(ncid, v6id, 'units', NCCHAR, 2, 'T0', iret)
       call ncaptc(ncid, v6id, 'long_name', NCCHAR, 1, 't', iret)
       call ncaptc(ncid, v7id, 'units', NCCHAR, 2, 'S0', iret)
       call ncaptc(ncid, v7id, 'long_name', NCCHAR, 1, 's', iret)
      endif
! global attributes
      call ncaptc(ncid,NCGLOBAL,'DATA_TYPE',NCCHAR,5,'MODEL',iret)
      call ncaptc(ncid,NCGLOBAL,'COORD_SYSTEM',NCCHAR,5,'LOCAL',iret)
! leave define mode
      call ncendf(ncid, iret)
! store x
      start(1) = 1
      count(1) = NX
      do i=1,NX
       x(i)=(float(i-1)-0.5)*dx
      enddo
      call ncvpt(ncid, xid, start, count, x, iret)
! store y
      start(1) = 1
      count(1) = NY
      do j=1,NY
       y(j)=(float(j-1)-0.5)*dy
      enddo
      call ncvpt(ncid, yid, start, count, y, iret)
! store z
      start(1) = 1
      count(1) = NZ+2
      z(1)=0.
      z(NZ+2)=1.
      do k=2,NZ+1
       z(k)=(float(k-1)-0.5)*dz
      enddo
      call ncvpt(ncid, zid, start, count, z, iret)
! store time
      mindex1(1) = 1
      time=itc
      call ncvpt1(ncid, timeid, mindex1, time, iret)
!  Store values in ncdf file
!  Store rho (top and bottom first, then interior with boundary)
      start4(1)=2
      start4(2)=2
      start4(3)=1
      start4(4)=1
      count4(1)=NX-2
      count4(2)=NY-2
      count4(3)=1
      count4(4)=1
      do 10 j=2,NY-1
       do 10 i=2,NX-1
        v0(i-1+(j-2)*(NX-2))=sngl(r(i,j,0))
10    continue
      call ncvpt(ncid, v1id, start4, count4, v0, iret)
      start4(1)=2
      start4(2)=2
      start4(3)=NZ+2
      start4(4)=1
      count4(1)=NX-2
      count4(2)=NY-2
      count4(3)=1
      count4(4)=1
      do 12 j=2,NY-1
       do 12 i=2,NX-1
        v0(i-1+(j-2)*(NX-2))=sngl(r(i,j,NZ+1))
12    continue
      call ncvpt(ncid, v1id, start4, count4, v0, iret)
      start4(1)=1
      start4(2)=1
      start4(3)=2
      start4(4)=1
      count4(1)=NX
      count4(2)=NY
      count4(3)=NZ
      count4(4)=1
      do 14 k=1,NZ
       do 14 j=1,NY
        do 14 i=1,NX
         v0(i+(j-1)*NX+(k-1)*NY*NX)=sngl(r(i,j,k))
14    continue
      call ncvpt(ncid, v1id, start4, count4, v0, iret)
!  Store p, p0
      start4(1)=1
      start4(2)=1
      start4(3)=2
      start4(4)=1
      count4(1)=NX
      count4(2)=NY
      count4(3)=NZ+1
      count4(4)=1
      do 20 k=1,NZ
       do 20 j=1,NY
        do 20 i=1,NX
         v0(i+(j-1)*NX+(k-1)*NY*NX)=sngl(p(i,j,k))
20    continue
      do 22 j=1,NY
       do 22 i=1,NX
        v0(i+(j-1)*NX+NZ*NY*NX)=sngl(p0(i,j))
22    continue
      call ncvpt(ncid, v2id, start4, count4, v0, iret)
!  Store u,v -- interpolate to rho points
      start4(1)=2
      start4(2)=2
      start4(3)=2
      start4(4)=1
      count4(1)=NX-2
      count4(2)=NY-2
!      count4(3)=NZ+iekman
      count4(3)=NZ+1
      count4(4)=1
      do 30 k=1,NZ
       do 30 j=2,NY-1
        do 30 i=2,NX-1
         v0(i-1+(j-2)*(NX-2)+(k-1)*(NY-2)*(NX-2))=
     .          0.5*sngl(u(i,j,k)+u(i-1,j,k))
30    continue
      if(iekman.eq.2) then
       do 32 j=2,NY-1
        do 32 i=2,NX-1
         v0(i-1+(j-2)*(NX-2)+NZ*(NY-2)*(NX-2))=0.5*(ue(i,j)+ue(i-1,j))
32     continue
      endif
      call ncvpt(ncid, v3id, start4, count4, v0, iret)
      do 40 k=1,NZ
       do 40 j=2,NY-1
        do 40 i=2,NX-1
         v0(i-1+(j-2)*(NX-2)+(k-1)*(NY-2)*(NX-2))=
     .          0.5*sngl(v(i,j,k)+v(i,j-1,k))
40    continue
      if(iekman.eq.2) then
       do 42 j=2,NY-1
        do 42 i=2,NX-1
         v0(i-1+(j-2)*(NX-2)+NZ*(NY-2)*(NX-2))=0.5*(ve(i,j)+ve(i,j-1))
42     continue
      endif
      call ncvpt(ncid, v4id, start4, count4, v0, iret)
!  Store w -- interpolate interior values to rho points
      start4(1)=2
      start4(2)=2
      start4(3)=1
      start4(4)=1
      count4(1)=NX-2
      count4(2)=NY-2
      count4(3)=NZ+2
      count4(4)=1
!  Store w top and bottom
      do 50 j=2,NY-1
       do 50 i=2,NX-1
        v0(i-1+(j-2)*(NX-2))=sngl(w(i,j,0))
        v0(i-1+(j-2)*(NX-2)+(NZ+1)*(NY-2)*(NX-2))=sngl(w(i,j,NZ+1))
!   Reset top w for vertical average below
        w(i,j,NZ)=w(i,j,NZ+1)
50    continue
      do 52 k=1,NZ
       do 52 j=2,NY-1
        do 52 i=2,NX-1
         v0(i-1+(j-2)*(NX-2)+k*(NY-2)*(NX-2))=
     .      0.5*sngl(w(i,j,k)+w(i,j,k-1))
52    continue
      call ncvpt(ncid, v5id, start4, count4, v0, iret)
!  Store t, s 
      if(isal.eq.1) then
       start4(1)=2
       start4(2)=2
       start4(3)=1
       start4(4)=1
       count4(1)=NX-2
       count4(2)=NY-2
       count4(3)=NZ+2
       count4(4)=1
       do 60 k=0,NZ+1
        do 60 j=2,NY-1
         do 60 i=2,NX-1
          v0(i-1+(j-2)*(NX-2)+k*(NY-2)*(NX-2))=sngl(t(i,j,k))
60     continue
       call ncvpt(ncid, v6id, start4, count4, v0, iret)
       do 70 k=0,NZ+1
        do 70 j=2,NY-1
         do 70 i=2,NX-1
          v0(i-1+(j-2)*(NX-2)+k*(NY-2)*(NX-2))=sngl(s(i,j,k))
70     continue
       call ncvpt(ncid, v7id, start4, count4, v0, iret)
      endif
!  Close ncdf file
      call ncclos(ncid,iret)
      stop
      end

