      subroutine init1(t,to,s,so,w,r,p0,p,u,v,tt)
! This routine may be changed by the user. 
!   Sets w_E, ts, ss and initialize dynamical arrays (t,to,s,so)

!  Variables changed on output: t,to,s,so,w,r,p0,p

! **It is recommended that the user examine this routine to ensure
!   it provides what is required.**
!
! Various values of the surface
! temperature and salinity field are set here. These must be 
! changed, depending on how the model is configured - e.g. cartesian
! vs spherical, global vs hemispheric etc, and then the model
! recompiled. 
  
      include 'params.inc'
      include 'common.inc'
      real t(NX,NY,0:NZ+1),to(NX,NY,0:NZ+1)
     .,    s(NX,NY,0:NZ+1),so(NX,NY,0:NZ+1)
      real w(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1),p0(NX,NY),p(NX,NY,NZ+1)
      real u(NX,NY,NZ+1),v(NX,NY,NZ+1)
      real tt(NX,NY,NZ+1)
 
! This routine currently
! implements a simple analytic wind field, independent of 
! the x-coordinate. If the windfield is changed, the primary
! restriction of the current model is that the Ekman pumping velocity
! integrate to zero. This is required in order to satisfy 
! global mass conservation, as lateral normal velocity is imposed
! to be zero at the boundaries. The elliptic solver will not find
! a solution if this condition is not satisfied. 

! Further, the model is currently set up to use a constant wind. 
! If a time-dependent wind is desired, the barotropic streamfunction
! must be recalculated each timestep (or at least from time to time). 
! This could be implemented with a modicum of recoding. 

! Write out wind field that is used: 
! Alwe is the number of gyres, set in const.name

      write(6,'('' init1: we=we0*cos(alwe*pi*y), alwe='',f4.1)') alwe
      pi=atan2(0.,-1.)            ! set pi to machine accuracy


! Now set velocities (ue, ve, we) in Ekman layer. 
!   Here we take tau proportional to f, so net vertical flux is zero
!        iekman = 0 keeps Ekman velocities ue, ve but neglects advection
!                  by geostrophic flow at top of interior
!        iekman = 1 keeps both Ekman velocity and geostrophic
!                   velocity in Ekman layer.

! If the wind is changed, ue, ve, and we must be changed consistently.
      do j=1,NY
       y=ys+(float(j-1)-0.5)*dy   ! ys is set in const.name
       do i=1,NX         
        ue(i,j)=0.     
        ve(i,j)=(we0*rde*aly/(alwe*pi))*scv(j)
     .          *sin(alwe*pi*(y+0.5*dy-ys)/aly)
!    we=de*ve_y    (de is Ekman layer thickness). 
        we(i,j)=we0*cos(alwe*pi*(y-ys)/aly)*scu(j)
        w0(i,j)=w00*(y-0.5)
        w(i,j,NZ+1)=we(i,j)+w0(i,j)
        w(i,j,NZ)=w(i,j,NZ+1)
       enddo
      enddo

!  Check area integral of we is zero (need this for elliptic solver)
      wea=0.
      do j=2,NY-1
       do i=2,NX-1
        wea=wea+we(i,j)/scu(j)
       enddo
      enddo
      wea=wea/float((NX-2)*(NY-2))
      write(6,'('' wea= '',e10.4)') wea

! Now set surface temperature and salinity fields. These determine
! the surface fluxes of heat and salinity. Can be changed by the user.

!  If yt0 is northern edge of subtropical gyre then alwe*pi*yt0=3*pi/2,
!  then:
!      yt0=3./(2.*alwe)
!    if yt0 is southern boundary, yt0=0
!      yt0=0.

! The scheme below implements a uniform gradient of temperature
! and salinity across the domain, with no zonal dependence.  
      ts1=ts1/aly
      ts2=ts2/aly
      do j=1,NY
       y=ys+(float(j-1)-0.5)*dy 
        do i=1,NX
!*********************************************************************
! inserted by S.-K. Lee (May/19/2009)
!        if(y.le.0.25) then
!         ts(i,j)=ts1
!        endif
!        if(y.ge.0.75) then
!         ts(i,j)=ts2
!        endif
!        if(y.gt.0.25.and.y.lt.0.75) then
!         ts(i,j)=ts1 + (ts2-ts1)*(y-0.25)/0.5
!        endif
!*********************************************************************
         ts(i,j)= 0.5*(ts2-ts1)*(y-0.5)/0.5
!*********************************************************************
!        if(y.le.yt0) then
!         ts(i,j)=ts1*y
!        else
!         ts(i,j)=ts1*yt0+ts2*(y-yt0)
!        endif
!*********************************************************************
         ss(i,j)=ss0*y
       enddo
      enddo

! Following lines left in for historical reasons
!c      do j=2,NY-1
!c       do i=2,NX-1
!c        we(i,j)=we(i,j)-wea
!c       enddo
!c      enddo
!c  Get ve=-Int(w_z;dy)
!c      do 30 j=2,NY-1
!c       ve(1,j)=ve(1,j-1)+0.5*dy*(we(2,j)+we(2,j-1))/de
!c       do 30 i=2,NX
!c        ve(i,j)=ve(1,j)
!c30    continue

!  Initialize temperature and salinity arrays
      if(ifile.eq.' ') then  ! set temperatures here
        do k=1,NZ
         do j=1,NY
          do i=1,NX
           t(i,j,k)=tb0
           s(i,j,k)=sb0
          enddo
         enddo
        enddo

        do j=1,NY
         do i=1,NX
          t(i,j,NZ+1)=ts(i,j)
          s(i,j,NZ+1)=ss(i,j)
          t(i,j,0)=tb0
          s(i,j,0)=sb0
         enddo
        enddo

       !  Get first guess pressure
        do j=1,NY
         do i=1,NX
          p0(i,j)=0.
         enddo
        enddo

      else             ! temperature and salinity previously
                       ! read from file. set surface values from these. 
        do i=1,NX
         t(i,1,NZ+1)=ts(i,1)
         s(i,1,NZ+1)=ss(i,1)
         t(i,NY,NZ+1)=ts(i,NY)
         s(i,NY,NZ+1)=ss(i,NY)
        enddo
        do j=2,NY-1
          t(1,j,NZ+1)=ts(1,j)
          s(1,j,NZ+1)=ss(1,j)
          t(NX,j,NZ+1)=ts(NX,j)
          s(NX,j,NZ+1)=ss(NX,j)
        enddo

      endif

!  Set previous timestep values for use in pgstep

      do k=0,NZ+1
       do j=1,NY
        do i=1,NX
         to(i,j,k)=t(i,j,k)
         so(i,j,k)=s(i,j,k)
        enddo
       enddo
      enddo

      if(ipr.eq.2) then             ! some printouts
       write(6,'('' t0:'',/,(100e10.4))') t
       write(6,'('' s0:'',/,(100e10.4))') s
      endif

!   Now solve barotropic elliptic problem
      call ell2d(p0,tt,w(1,1,NZ+1),w(1,1,0))

      return
      end
!
! ----------------------------------------------------------------
!
      subroutine gridz(z,adz,de)
! This routine sets up the vertical grid. 
! Input parameter: adz
! Array changed on output: z.

! If adz not equal to 100 or 101, then a standard stretching
! formula is used to give more resolution near the surface. 
! adz then determines the amount of stretching. A value of adz=3
! will work for many purposes. Adz is set in const.name

! If adz=100 a different grid is used for a higher 
! vertical resolution model (NZ = 64).
! If adz=101 the values are entered 'by hand' below. 

      include 'params.inc'
      real z(0:NZ+1)
      if(adz.eq.100.) then
!  IBL (internal boundary layer) vertical grid for 65^3 (NZ=64)
       k0=12
       k1=52
       sigma0=0.65
       sigma1=1.1
       z(0)=0.
       dz=1.
       z(1)=0.5*dz
       do k=2,k0
        dz=sigma0*dz
        z(k)=z(k-1)+dz
       enddo
       do k=k0+1,k1
        z(k)=z(k-1)+dz
       enddo
       do k=k1+1,NZ
!       do k=k1+1,NZ-1
        dz=sigma1*dz
        z(k)=z(k-1)+dz
       enddo
       z(NZ)=z(NZ-1)+dz
       znorm=z(NZ)
!       z(NZ+1)=z(NZ)+0.5*dz
!       znorm=z(NZ+1)
       do k=0,NZ
        z(k)=z(k)/znorm
       enddo
       z(NZ+1)=1.+de
      else if(adz.eq.101.) then         !  GFDL 15-level grid (NZ=15)
!     else if(adz.eq.101.) then         !  GFDL 15-level grid (NZ=14)
! following are dimensional depths used in a particular MOM comparison
       z(16)=  0.
       z(15)= 30.
       z(14)= 53.15
       z(13)=110.02
       z(12)=190.98
       z(11)=302.44
       z(10)=451.96
       z(9)= 648.20
       z(8)= 900.90
       z(7)=1220.8
       z(6)=1619.2
       z(5)=2107.8
       z(4)=2698.5
       z(3)=3402.3
       z(2)=4229.1
       z(1)=5187.2
       z(0)=5700.
! now convert to non-dimensional heights between 0 and 1. 
       do k=0,16
!       do k=0,15
        z(k)=(5700.-z(k))/5670.
       enddo
       de = z(NZ+1) - z(NZ)
!*********************************************************************
! inserted by S.-K. Lee (Jul/06/2009)
      else if(adz.eq.1.) then         !  GFDL 15-level grid (NZ=15)
       z(4)=   0.0
       z(3)=  30.0
       z(2)=1447.5
       z(1)=4282.5
       z(0)=5700.0
! now convert to non-dimensional heights between 0 and 1. 
       do k=0,4
        z(k)=(5700.-z(k))/5670.
       enddo
       de = z(NZ+1) - z(NZ)
!*********************************************************************
      else     !  adz is now the vertical stretching parameter
       z(0)=0.
       z(NZ+1)=1.+de
       do  k=1,NZ
        z(k)=log(1.+float(2*k-1)*(exp(adz)-1)/float(2*NZ))/adz
       enddo
!  Shift so z(NZ)=1
       z(NZ)=0.5*(z(NZ)+z(NZ-1))
       ddz=1.-z(NZ)
       z(NZ)=1.
       do k=1,NZ-1
        z(k)=z(k)+ddz
       enddo
      endif
! All done 
      return
      end
!
! ----------------------------------------------------------------
!
      subroutine initlz(time,t,to,s,so,r,p0,p,u,v,w,tt,st)
!  Set constants and initializes dynamical arrays
! All arrays are initialized here. This routine not normally changed by
! the user. 
      include 'params.inc'
      include 'common.inc'
      real t(NX,NY,0:NZ+1),to(NX,NY,0:NZ+1)
     .,    s(NX,NY,0:NZ+1),so(NX,NY,0:NZ+1)
      real r(NX,NY,0:NZ+1),p0(NX,NY),p(NX,NY,NZ+1)
     .,    u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)
      real tt(NX,NY,NZ+1),st(NX,NY,NZ+1)

! Declare the namelist
      namelist /constn/ ifile,ofile
     ., isal,iconv,iekman,ispher,ipr
     ., nt,t0,dt,irk,alx,aly,adz,ys,f0,beta
     ., akvt,akht,alht,akvs,akhs,alhs
     ., gammats,gammatb,gammat0
     ., gammass,gammasb,gammas0
     ., ts1,ts2,yt0,tb0,ss0,sb0,scap0
     ., we0,alwe,de,w00
     ., epsi,epsb,depsb,epse,depse
     ., omega1,tol1,maxits
! Read the namelist
      lun=10
      open(unit=lun,file='const.name',status='OLD')
      read(lun,constn)
      rewind lun
! The meanings of the variables in the namelist are to be found in
! the README file accompanying the code. The namelist contains
! most of the variables that need to changed frequently. 

! Now write out the values of the parameters to the terminal,
! or to a log file:
      write(6,'('' pgom:  ifile,ofile='',2(2x,a6))') ifile,ofile
      write(6,'(''      NX,NY,NZ='',3i6)') NX,NY,NZ
      write(6,'(''     isal,iconv,iekman,ispher,ipr,irk,nt='',6i2,i10)')
     .                 isal,iconv,iekman,ispher,ipr,irk,nt
      write(6,'(''      t0,dt,alx,aly,adz,ys,f0,beta='')')
      write(6,'(2e12.5,6f7.3)') t0,dt,alx,aly,adz,ys,f0,beta
      write(6,'(''      akvt,akht,alht,akvs,akhs,alhs='')')
      write(6,'(6e12.5)') akvt,akht,alht,akvs,akhs,alhs
      write(6,
     . '(''      gammats,gammatb,gammat0,gammass,gammasb,gammas0='')')
      write(6,'(6e12.5)') 
     .           gammats,gammatb,gammat0,gammass,gammasb,gammat0
      write(6,'(''      ts1,ts2,yt0,tb0,ss0,sb0='')')
      write(6,'(6e12.5)') ts1,ts2,yt0,tb0,ss0,sb0
      write(6,'(''      we0,alwe,de,w00,scap0='')')
      write(6,'(5e12.5)') we0,alwe,de,w00,scap0
      write(6,'(''      epsi,epsb,depsb,epse,depse='')')
      write(6,'(5e12.5)') epsi,epsb,depsb,epse,depse
      write(6,'(''      omega1,tol1,maxits='',2e12.5,i8)')
     .                  omega1,tol1,maxits

!  Set numerical and physical constants (see README file). 
      pi=atan2(0.,-1.)
      if(ispher.eq.1) then    ! spherical option
       ys=ys*pi/180.
       alx=alx*pi/180.
       aly=aly*pi/180.
       depsb=depsb*pi/180.
       depse=depse*pi/180.
      endif
      dx=alx/float(NX-2)
      dy=aly/float(NY-2)
      rdx=1./dx
      rdy=1./dy
      r2dx=0.5/dx
      r2dy=0.5/dy
! set up vertical grid
      call gridz(z,adz,de)           
      dzw(0)=z(1)-z(0)
      dzw(1)=z(2)-z(1)
      dzw(NZ)=z(NZ+1)-z(NZ)
      dzr(1)=0.5*(z(2)+z(1))-z(0)
      dzr(NZ)=z(NZ)-0.5*(z(NZ)+z(NZ-1))
      do 10 k=2,NZ-1
       dzw(k)=z(k+1)-z(k)
       dzr(k)=0.5*(z(k+1)-z(k-1))
10    continue
      rdzw(0)=1./dzw(0)
      do 12 k=1,NZ
       rdzw(k)=1./dzw(k)
       rdzr(k)=1./dzr(k)
       r2dzr(k)=0.5*rdzr(k)
12    continue

      rde=1./de

! Spherical or Cartesian geometries
      if(ispher.eq.1) then     ! spherical option
       omgcap=1.
       do j=1,NY
        f(j)=2.*omgcap*sin(ys+dy*float(j-1)-0.5*dy)
        f1(j)=2.*omgcap*sin(ys+dy*float(j-1))
        scu(j)=1./cos(ys+dy*float(j-1)-0.5*dy)
        csv(j)=cos(ys+dy*float(j-1))
        scv(j)=1./csv(j)
       enddo
!  Compute default alht, alhs using phi=35deg to estimate beta
!    in wbc for spherical case? 
!       if(alht.lt.0.) alht=akht*(epsb/(2.*omgcap*cos(35.*pi/180.)))**2
!       if(alhs.lt.0.) alhs=akhs*(epsb/(2.*omgcap*cos(35.*pi/180.)))**2
      else                      ! Cartesian option
       do j=1,NY
        f(j)=f0+beta*dy*(float(j-1)-0.5*float(NY-1))
        f1(j)=f0+beta*dy*(float(j-1)-0.5*float(NY-1)+0.5)
        scu(j)=1.
        csv(j)=1.
        scv(j)=1.
       enddo
!  Compute default alht, alhs for Cartesian geometry
       if(alht.lt.0.) alht=akht*(epsb/beta)**2
       if(alhs.lt.0.) alhs=akhs*(epsb/beta)**2
      endif
      write(6,'(''  alht,alhs='',2e15.6)') alht,alhs
!  Zero arrays
       do k=1,NZ+1
        do j=1,NY
         do i=1,NX
          r(i,j,k)=0.
          s(i,j,k)=0.
          t(i,j,k)=0.
          u(i,j,k)=0.
          v(i,j,k)=0.
          w(i,j,k)=0.
          p(i,j,k)=0.
          tt(i,j,k)=0.
          st(i,j,k)=0.
          so(i,j,k)=0.
          to(i,j,k)=0.
         enddo
        enddo
       enddo
      do j=1,NY
       do i=1,NX
        r(i,j,0)=0.
        s(i,j,0)=0.
        t(i,j,0)=0.
        w(i,j,0)=0.
        p0(i,j)=0.
        so(i,j,0)=0.
        to(i,j,0)=0.
       enddo
      enddo
        
! Set initial time
      time=t0
!  Initialize elliptic solver
      call eparams(tt)

!  Initialize dynamical arrays
    
! Get initial thermocline density and first guess barotropic pressure.
! The t and s fields are read in from a data file if 'ifile'
!  is not equal to a single blank character.  Otherwise the initial
!  fields are set in init1
      if(ifile.ne.' ') call pgin(t,s,r,p0) 
!   Set initial t and s and forcing (w) fields; get p0 and p
      call init1(t,to,s,so,w,r,p0,p,u,v,tt)

      return
      end
!
! ----------------------------------------------------------------
!
      subroutine diags(t,s,r,p,p0,u,v,w,rhosfc,rhotot,hketot)
! Mean surface density, total mass & horizontal kinetic energy
! This is a diagnostic routine only and its results do
! not affect the model evolution. 

! Inputs: t,s,r,p,p0,u,v,w
! Variables changed on output: rhosfc,rhotot,hketot0
! Uses common blocks. 
      include 'params.inc'
      include 'common.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1),r(NX,NY,0:NZ+1)
      real p(NX,NY,NZ+1),p0(NX,NY)
      real u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)
!
! Calculate mean surface density
!
      rhosfc=0.
      do 10 j=2,NY-1
       do 10 i=2,NX-1
        rhosfc=rhosfc-t(i,j,NZ+1)*(1./scu(j))
10    continue
      rhosfc=rhosfc/((NX-2)*(NY-2))
!
! Calculate total mass & horizontal kinetic energy
!
      rhotot=0.
      hketot=0.
      do 100 k=1,NZ
!  Mass (Heat content)
       do 30 j=2,NY-1
        do 30 i=2,NX-1
        rhotot=rhotot-t(i,j,k)*dzr(k)/scu(j)
30     continue
!  HKE
       do 50 j=2,NY-2
        do 50 i=2,NX-2
         hketot=hketot+(u(i,j,k)**2+v(i,j,k)**2)*dzr(k)/scu(j)
50     continue
       do 52 i=2,NX-2
        hketot=hketot+u(i,NY-1,k)**2*dzr(k)/scu(NY-1)
52     continue
       do 54 j=2,NY-2
        hketot=hketot+v(NX-1,j,k)**2*dzr(k)/scu(j)
54     continue
100   continue
      rhotot=rhotot*dx*dy
! Include Ekman layer mass in total mass
      rhotot=rhotot+rhosfc*de*alx*aly
      hketot=0.5*hketot*dx*dy
      return
      end
!
! ----------------------------------------------------------------
!
