      subroutine pguvw(tt,t,s,r,p0,p,u,v,w)
! Routine  calculates horizontal and vertical velocities from r, p0
! Input variables:  t, s, r, p0, p
! Variables changed on output: u, v, w
! Also uses common blocks.

      include 'params.inc'
      include 'common.inc'
      real t(NX,NY,0:NZ+1),s(NX,NY,0:NZ+1)
      real r(NX,NY,0:NZ+1),p0(NX,NY),p(NX,NY,NZ+1)
     .,    u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)

! Following few lines may need to be un-commented if winds vary
!c  Calculate thermocline horizontal velocities
!c   Solve elliptic equation for new barotropic pressure
!c    (Not needed for steady forcing)
!c      call ell2d(p0,work,w(1,1,NZ+1),w(1,1,0))
!      if(iekman.eq.2) call ell2d(p0,work,w(1,1,NZ+1),w(1,1,0))

!  Convectively adjust if statically unstable
      if(iconv.eq.2) call convct(t,s)  
!
!  Evaluate boundary conditions, currently same for T and S:
      call boundb(t)
      if(isal.eq.1) call boundb(s)
!
!  Calculate bottom (z=0) temperature and salinity
!   for advection/diffusion using flux condition
      call fluxbc(t,akvt,gammatb,tb0)
      if(isal.eq.1) call fluxbc(s,akvs,gammasb,sb0)
!
!  Evaluate equation of state for density
!    (Currently linear equation of state with no pressure dependence)
      call dents(t,s,r)
!
!  Get boundary density and integrate hydrostatic relation
      call vint(p,p0,r)
!
!  Calculate u,v from pressure; integrate continuity equation for w
      call velo(u,v,w,p)
!
! All done
      return
      end
!
! ----------------------------------------------------------------
!
      subroutine vint(p,p0,r)
! This routine solves for the pressure field, given the density
! field and the vertically integrated (barotropic) pressure field.  
      include 'params.inc'
      include 'common.inc'

      real r(NX,NY,0:NZ+1),p0(NX,NY),p(NX,NY,NZ+1)
! Boundary values of r are set in routine 'dents' from the boundary
!  values of t (and s, if isal=1).  We need pressure at these points
!  to calculate tangential velocities next to the boundary.

! Could do it all with one loop over (i,j), but the following is faster.

! First do the vertical hydrostatic integration, assuming zero pressure
!  at the bottom.  (The top and bottom densities are not used.)
      do j=1,NY
       do i=1,NX
        p(i,j,1)=-r(i,j,1)*dzw(0)
       enddo
      enddo
      do k=2,NZ
       do j=1,NY
        do i=1,NX
         p(i,j,k)=p(i,j,k-1)-(r(i,j,k)+r(i,j,k-1))*0.5*dzw(k-1)
        enddo
       enddo
      enddo
      do j=1,NY
       do i=1,NX
        p(i,j,NZ+1)=p(i,j,NZ)
       enddo
      enddo
!
! The pressure field is wrong by a constant factor. We now
! use the barotropic field to correct this:
      do 200 j=1,NY
       do 200 i=1,NX
        ssum=p(i,j,1)*dzr(1)
        do k=2,NZ
         ssum=ssum+p(i,j,k)*dzr(k)
        enddo
        if(iekman.eq.1) ssum=ssum+p(i,j,NZ+1)*dzw(NZ)
! This sum differs from the proper barotropic field:
        delta=(p0(i,j)-ssum)
        do k=1,NZ+1
         p(i,j,k)=p(i,j,k)+delta
        enddo
200   continue
! The three-dimensional pressure field is now contained in p(i,j,k)
!   at rho points, where k=1,NZ, and at z=1 (k=NZ+1)
!  Note that p(i,j,NZ+1) = p(i,j,NZ)
      return
      end
!
! ----------------------------------------------------------------
!
      subroutine velo(u,v,w,p)
! This routine solves for the velocity field, given the pressure
! field at rho points.  
! Note that forcing vertical velocity is kept in w(i,j,NZ+1),
! and is fixed.
! The field w(i,j,NZ) is the vertical velocity at the bottom
! of the boundary layer.
!  (For iekman = 0, w(i,j,NZ+1) and  w(i,j,NZ) should be equal)

! Input variable: p 
! Variables changed on output: u,v,w

      include 'params.inc'
      include 'common.inc'
      real p(NX,NY,NZ+1),u(NX,NY,NZ+1),v(NX,NY,NZ+1),w(NX,NY,0:NZ+1)
!
! First calculate the horizontal velocity fields, in the form used
!  in the differencing of the elliptic equation. 
      do 200 k=1,NZ+1
       do j=2,NY-1
        do i=1,NX-1
         u(i,j,k)=-0.25*g1u(i,j)*rdy*( p(i+1,j+1,k)+p(i,j+1,k) 
     .                                -p(i+1,j-1,k)-p(i,j-1,k) )
     .            -g2u(i,j)*scu(j)*rdx*(p(i+1,j,k)-p(i,j,k))
        enddo
       enddo
       do j=1,NY-1
        do i=2,NX-1
         v(i,j,k)=0.25*g1v(i,j)*scv(j)*rdx*( p(i+1,j+1,k)+p(i+1,j,k)
     .                                      -p(i-1,j+1,k)-p(i-1,j,k) )
     .                -g2v(i,j)*rdy*(p(i,j+1,k)-p(i,j,k))
        enddo
       enddo
200   continue
!
! Now evaluate w=-integral(u_x+v_y;dz), starting at the bottom
! 
      do k=1,NZ
       do j=2,NY-1
        do i=2,NX-1
         w(i,j,k)=w(i,j,k-1)
     . -dzr(k)*scu(j)*((u(i,j,k)-u(i-1,j,k))*rdx
     .                +(v(i,j,k)*csv(j)-v(i,j-1,k)*csv(j-1))*rdy)
        enddo
       enddo
      enddo
!
! Following few lines are historical.
!c Check top velocities
!      wdiff=0.
!      do 500 j=2,NY-1
!       do 500 i=2,NX-1
!        wdiff=max(wdiff,abs((w(i,j,NZ+1)-w(i,j,NZ))/w(i,j,NZ)))
! 500   continue
!      write(6,'('' wdiff= '',e9.3)') wdiff
!	print *, 'wdiff = ',wdiff
!      write(70,'(1x,e9.3)') wdiff
!
! Set Ekman layer velocities
!  Add geostrophic to Ekman velocity if iekman=1.
      if(iekman.eq.0) then
       do j=2,NY-1
        do i=1,NX-1
         u(i,j,NZ+1)=ue(i,j)
        enddo
       enddo
       do j=1,NY-1
        do i=2,NX-1
         v(i,j,NZ+1)=ve(i,j)
        enddo
       enddo
      else if(iekman.eq.1) then
       do j=2,NY-1
        do i=1,NX-1
         u(i,j,NZ+1)=u(i,j,NZ+1)+ue(i,j)
        enddo
       enddo
       do j=1,NY-1
        do i=2,NX-1
         v(i,j,NZ+1)=v(i,j,NZ+1)+ve(i,j)
        enddo
       enddo
      else
       print *, 'check value of iekman'
       stop
      endif
      return
      end
!
! 
! --------------------------------------------------------------------
!
      subroutine eparams(tt)
! Ths routine sets some parameters for the thermocline elliptic solver.
! All the arrays and paramters are time independent, and are held in
! common block /paramet/.
! This routine also calls bound1 to set up the matrix arrays for
!  the boundary solver
      include 'params.inc'
      include 'common.inc'
      real tt(1)
!
!  Set up variable eps arrays g1u,g2u,g1v,g2v
!   epsi        -- interior eps
!   epsb, depsb -- boundary eps
!   epse, depse -- equatorial (currently y-midpoint) eps
!
      do j=1,NY
       yu=(float(j-1)-0.5)*dy
       yeq=abs(yu-0.5*aly)
       do i=1,NX-1
        xu=float(i-1)*dx
        xyu=max(0.,min(xu,alx-xu,yu,aly-yu))
        if(xyu.lt.depsb) then
         eps1=epsi+(epsb-epsi)*((depsb-xyu)/depsb)**2
        else
         eps1=epsi
        endif
!        if(i.eq.1 .or. i.eq.NX-1) eps1=epsb
        eps1=eps1+max(epse-eps1,0.)*exp(-0.5*yeq**2/depse**2)
        g1u(i,j)=f(j)/(eps1**2+f(j)**2)
        g2u(i,j)=eps1/(eps1**2+f(j)**2)
       enddo
      enddo
      do j=1,NY-1
       yv=float(j-1)*dy
       yeq=abs(yv-0.5*aly)
       do i=1,NX
        xv=(float(i-1)-0.5)*dx
        xyv=max(0.,min(xv,alx-xv,yv,aly-yv))
        if(xyv.lt.depsb) then
         eps1=epsi+(epsb-epsi)*((depsb-xyv)/depsb)**2
        else
         eps1=epsi
        endif
!        if(j.eq.1 .or. j.eq.NY-1) eps1=epsb
        eps1=eps1+max(epse-eps1,0.)*exp(-0.5*yeq**2/depse**2)
        g1v(i,j)=f1(j)/(eps1**2+f1(j)**2)
        g2v(i,j)=eps1/(eps1**2+f1(j)**2)
       enddo
      enddo
!
!  The g1,g2 are redundant with g1u,g2u,g1v,g2v but leave in
!   for convenience
!
      do j=1,NY-1
        g1(2*j-1)=g1u(1,j)
        g2(2*j-1)=g2u(1,j)
        g1(2*j)=g1v(1,j)
        g2(2*j)=g2v(1,j)
      enddo
      g1(2*NY-1)=g1u(1,NY)
      g2(2*NY-1)=g2u(1,NY)
!
!  Call bound1 with icall=0 to initialize the matrix a,b,c
!  (The second argument is a dummy for icall=0)
!
      call bound1(0,tt,g1,g2,scu,scv,rdx,rdy)
      return
      end

!
! ----------------------------------------------------------------
!
      subroutine bound1(icall,f,g1,g2,scu,scv,rdx,rdy)

! This routine gets the boundary values of density, salinity 
! or pressure, for a particular layer. 

! Routine sets up various the arrays
! and then calls the periodic tridiagonal solver, subroutine cyclic.
! The arrays a(), b(), c() have their conventional meaning
! in tridiagonal matrices (see e.g. numerical recipes)
!  icall=0 => set up coefficient matrix a,b,c and save
!       =1 => set up source vector s and solve

! Input variables: icall,f,g1,g2,scu,scv,rdx,rdy
! Variables changed on output: f

      include 'params.inc'
      real f(NX,NY),s(KTOT),bf(KTOT),g1(2*NY-1),g2(2*NY-1)
      real a(KTOT),b(KTOT),c(KTOT)
      real scu(NY),scv(NY)
      save a,b,c 

      if(icall.eq.0) then
! start at lower left corner
      k=1
      a(k)=-g1(2)
      c(k)= g1(2)
      b(k)= g2(2)
      k=k+1
      a(k)=(-g1(2)*0.25-0.25*g2(2))*scv(1)*rdx
      c(k)= g1(2)*0.25*scv(1)*rdx
      b(k)= g2(2)*rdy     -0.25*g1(2)*scv(1)*rdx
      k=k+1
!
      do i=3,NX-2
       a(k)=-g1(2)*0.25*scv(1)*rdx
       c(k)= g1(2)*0.25*scv(1)*rdx
       b(k)= g2(2)*rdy
       k=k+1
      enddo
!
! now bottom right hand corner
      a(k)=-g1(2)*0.25*scv(1)*rdx
      c(k)=(g1(2)*0.25-0.25*g2(2))*scv(1)*rdx
      b(k)= g2(2)*rdy     +0.25*g1(2)*scv(1)*rdx
      k=k+1
      a(k)=-g1(2)
      c(k)= g1(2)
      b(k)= g2(2)
      k=k+1
      a(k)=(-g1(3)*0.25-0.25*g1(3)*g2(2)/g1(2))*rdy
      c(k)= g1(3)*0.25*rdy
      b(k)= g2(3)*scu(2)*rdx     -0.25*g1(3)*rdy
      k=k+1
!
! now up the right hand side
      do j=3,NY-2
       j2=2*j-1
       a(k)=-g1(j2)*0.25*rdy
       c(k)= g1(j2)*0.25*rdy
       b(k)= g2(j2)*scu(j)*rdx
       k=k+1
      enddo
!
! now top right hand corner
      j=NY-1
      j2=2*j-1
      a(k)=-g1(j2)*0.25*rdy
      c(k)=( g1(j2)*0.25-0.25*g1(j2)*g2(j2+1)/g1(j2+1))*rdy
      b(k)= g2(j2)*scu(j)*rdx     +0.25*g1(j2)*rdy
      k=k+1
      j=NY
      j2=2*j-2
      a(k)=-g1(j2)
      c(k)= g1(j2)
      b(k)= g2(j2)
      k=k+1
      a(k)=(-g1(j2)*0.25-0.25*g2(j2))*scv(NY-1)*rdx
      c(k)= g1(j2)*0.25*scv(NY-1)*rdx
      b(k)= g2(j2)*rdy     -0.25*g1(j2)*scv(NY-1)*rdx
      k=k+1
!
! now along the top	
      do i=NX-2,3,-1
       a(k)=-g1(j2)*0.25*scv(NY-1)*rdx
       c(k)= g1(j2)*0.25*scv(NY-1)*rdx
       b(k)= g2(j2)*rdy
       k=k+1
      enddo
! now top left hand corner
      a(k)=-g1(j2)*0.25*scv(NY-1)*rdx
      c(k)=( g1(j2)*0.25-0.25*g2(j2))*scv(NY-1)*rdx
      b(k)= g2(j2)*rdy     +0.25*g1(j2)*scv(NY-1)*rdx
      k=k+1
      a(k)=-g1(j2)
      c(k)= g1(j2)
      b(k)= g2(j2)
      k=k+1
!
      j=NY-1
      j2=2*j-1
      a(k)=(-g1(j2)*0.25-0.25*g1(j2)*g2(j2+1)/g1(j2+1))*rdy
      c(k)= g1(j2)*0.25*rdy
      b(k)= g2(j2)*scu(j)*rdx     -0.25*g1(j2)*rdy
      k=k+1
!
! now down the left hand side
      do j=NY-2,3,-1
       j2=2*j-1
       a(k)=-g1(j2)*0.25*rdy
       c(k)= g1(j2)*0.25*rdy
       b(k)= g2(j2)*scu(j)*rdx
       k=k+1
      enddo
      a(k)=-g1(3)*0.25*rdy
      c(k)=( g1(3)*0.25-0.25*g1(3)*g2(2)/g1(2) )*rdy
      b(k)= g2(3)*scu(2)*rdx     +0.25*g1(3)*rdy
      k=k+1
!
      else if(icall.eq.1) then
!
! start at lower left corner
      k=1
      s(k)= g2(2)*f(2,2)
      k=k+1
      s(k)= f(2,2)*g2(2)*rdy
     .    +( -0.25*g1(2)*f(3,2)-0.25*g2(2)*f(2,2) )*scv(1)*rdx
      k=k+1
!
      do i=3,NX-2
       s(k)= f(i,2)*g2(2)*rdy
     .       -0.25*g1(2)*(f(i+1,2)-f(i-1,2))*scv(1)*rdx
       k=k+1
      enddo
!
! now bottom right hand corner
      s(k)= f(NX-1,2)*g2(2)*rdy
     . +( -0.25*g1(2)*(-f(NX-2,2))-0.25*g2(2)*f(NX-1,2) )*scv(1)*rdx
      k=k+1
      s(k)= g2(2)*f(NX-1,2)
      k=k+1
      s(k)=f(NX-1,2)*g2(3)*scu(2)*rdx
     .  +( -0.25*g1(3)*f(NX-1,3)-0.25*g1(3)*g2(2)*f(NX-1,2)/g1(2) )*rdy
      k=k+1
!
! now up the right hand side
      do j=3,NY-2
       j2=2*j-1
       s(k)=f(NX-1,j)*g2(j2)*scu(j)*rdx
     +      -0.25*g1(j2)*(f(NX-1,j+1)-f(NX-1,j-1))*rdy
       k=k+1
      enddo
!
! now top right hand corner
      j=NY-1
      j2=2*j-1
      s(k)= f(NX-1,NY-1)*g2(j2)*scu(j)*rdx
     .     +(-0.25*g1(j2)*(-f(NX-1,NY-2))
     .       -0.25*g1(j2)*g2(j2+1)*f(NX-1,NY-1)/g1(j2+1))*rdy
      k=k+1
      j=NY
      j2=2*j-2
      s(k)= g2(j2)*f(NX-1,NY-1)
      k=k+1
      s(k)= f(NX-1,NY-1)*g2(j2)*rdy
     .    +(-0.25*g1(j2)*f(NX-2,NY-1)
     .      -0.25*g2(j2)*f(NX-1,NY-1))*scv(NY-1)*rdx
      k=k+1
!
! now along the top	
      do i=NX-2,3,-1
       s(k)=f(i,NY-1)*g2(j2)*rdy
     .      -0.25*g1(j2)*(f(i-1,NY-1)-f(i+1,NY-1))*scv(NY-1)*rdx
       k=k+1
      enddo
! now top left hand corner
      s(k)= f(2,NY-1)*g2(j2)*rdy
     .   +(-0.25*g1(j2)*(-f(3,NY-1))
     .     -0.25*g2(j2)*f(2,NY-1))*scv(NY-1)*rdx
      k=k+1
      s(k)= g2(j2)*f(2,NY-1)
      k=k+1
!
      j=NY-1
      j2=2*j-1
      s(k)= f(2,NY-1)*g2(j2)*scu(j)*rdx
     .      +(-0.25*g1(j2)*f(2,NY-2)
     .        -0.25*g1(j2)*g2(j2+1)*f(2,NY-1)/g1(j2+1))*rdy
      k=k+1
!
! now down the left hand side
      do j=NY-2,3,-1
       j2=2*j-1
       s(k)=f(2,j)*g2(j2)*scu(j)*rdx
     .      -0.25*g1(j2)*(f(2,j-1)-f(2,j+1))*rdy
       k=k+1
      enddo
      s(k)= f(2,2)*g2(3)*scu(2)*rdx
     .      +( -0.25*g1(3)*(-f(2,3))
     .         -0.25*g1(3)*g2(2)*f(2,2)/g1(2))*rdy
      k=k+1
!
      if(k.ne.KTOT+1)then
       print *,k,KTOT,i,j
       stop 'KTOT'
      endif
c source array set up. Now solve for boundary value:
      beta=a(1)
      alpha=c(KTOT)
      call cyclic(a,b,c,alpha,beta,s,bf,KTOT)    
c
C Now reconstruct boundary
      k=1
      do i=1,NX
       f(i,1)=bf(k)
      k=k+1
      enddo
      do j=2,NY
       f(NX,j)=bf(k)
       k=k+1
      enddo
      do i=NX-1,1,-1
       f(i,NY)=bf(k)
       k=k+1
      enddo
      do j=NY-1,2,-1
       f(1,j)=bf(k)
       k=k+1
      enddo
c
      endif
      return
      end

!
! ----------------------------------------------------------------
!

      subroutine boundb(r)

! This routine sets the boundary conditions by imposing the
!  no-normal flow condition on the incoming field. The incoming
!  field (r) will normally be temperature or salinity.  This routine
!  applies the boundary conditions to all model levels, except the
!  0th and (NZ+1)th layers.  To apply the boundary condition to one
!  layer only, call bound1 directly.

!     Input variable: r
!     Variable changed on output: r
!     uses common block.
!     Calls routine bound1
     
      include 'params.inc'
      include 'common.inc'
      real r(NX,NY,0:NZ+1)
      do k=1,NZ
       call bound1(1,r(1,1,k),g1,g2,scu,scv,rdx,rdy)
      enddo
      return
      end

!
! ----------------------------------------------------------------
!

      subroutine cyclic(a,b,c,alpha,beta,r,x,n)
! This is a cyclic tridiagonal solver. Should not be changed.
! Use as a black box. 
! (okay to increase value of NMAX for large integration).
!    Calls: tridag
!
      parameter (NMAX=500)
      integer n,i
      real alpha,beta,a(n),b(n),c(n),r(n),x(n)
      real fact,gamma,bb(NMAX),u(NMAX),z(NMAX)

      if(n.le.2)pause 'n too small in cyclic'
      if(n.gt.NMAX)pause 'NMAX too small in cyclic'

      gamma=-b(1)
      bb(1)=b(1)-gamma
      bb(n)=b(n)-alpha*beta/gamma
      do i=2,n-1
        bb(i)=b(i)
      enddo
      call tridag(a,bb,c,r,x,n)
      u(1)=gamma
      u(n)=alpha
      do i=2,n-1
        u(i)=0.
      enddo
      call tridag(a,bb,c,u,z,n)
      fact=(x(1)+beta*x(n)/gamma)/(1.+z(1)+beta*z(n)/gamma)
      do i=1,n
        x(i)=x(i)-fact*z(i)
      enddo 
      return
      end

!
! ----------------------------------------------------------------
!

      subroutine tridag(a,b,c,r,u,n)
!  Tridiagonal solver (e.g., Numerical Recipes, Press et al., 1992)
      parameter (NMAX=500)
      integer n,j
      real a(n),b(n),c(n),r(n),u(n)
      real bet,gam(NMAX)
      if(b(1).eq.0.)pause 'tridag: rewrite equations'
      rbet=1./b(1)
      u(1)=r(1)*rbet
!      bet=b(1)
      do j=2,n
        gam(j)=c(j-1)*rbet
        rbet=1./(b(j)-a(j)*gam(j))
        u(j)=(r(j)-a(j)*u(j-1))*rbet
! Original code had twice the divides, since it uses
!      bet=1./rbet
!  but could check:
!      if(bet.eq.0.)pause 'tridag failed'
      enddo
      do j=n-1,1,-1
        u(j)=u(j)-gam(j+1)*u(j+1)
      enddo
      return
      end

!
! ----------------------------------------------------------------
!
      subroutine ell2d(phi2,s2,wtop,wbot)
! This routine is the parent routine that calls the elliptic solving
! routines. ! It exists in part for historical reasons.
! Its functionality could be combined with that of routine 'esolv'. 

! Input variables:  wbot, wtop
! Variables changed on output: phi2, s2
      include 'params.inc'
      include 'common.inc'
      dimension phi2(NX,NY)
      dimension s2(NX,NY),wtop(NX,NY),wbot(NX,NY)
! First construct proper barotropic source:
      facm=0.
      do  i=2,NX-1
       do  j=2,NY-1
        s2(i,j)=(wbot(i,j) - wtop(i,j))
       enddo
      enddo
! Now call the elliptic solver.
      call esolv(phi2,s2,facm)
! Now check solution if ipr=1
!  The check & bcheck routines are incorrect for the current code
!      if(ipr.ge.1)then
!       call check(phi2,s2)
!       call bcheck(phi2)
!     endif
! All done
      return 
      end
!
! ----------------------------------------------------------------
!
      subroutine esolv(phi,s2,facm)
! This routine calls the actual elliptic solver 'sorell'. 
! Input variables: s2
! Variable changed on output: phi 
! Uses common blocks.
      include 'params.inc'
      include 'common.inc'
      dimension phi(NX,NY),s2(NX,NY)

      call sorell(facm,phi,g1u,g2u,g1v,g2v,scu,scv,csv,g1,g2,s2
     .,  rdx,rdy,NX,NY,KTOT,omega1,tol1,maxits,ipr)
! Now call bound1 one more time to ensure boundary conditions are
!   exactly satisfied. 
      call bound1(1,phi,g1,g2,scu,scv,rdx,rdy)  
      return
      end

      subroutine sorell(facm,f,g1u,g2u,g1v,g2v,scu,scv,csv,g1,g2,s
     .,   rdx,rdy,imax,jmax,KTOT,omega1,tol1,maxits,ipr)
! This routine does an SOR iteration to solve the elliptic problem for
!  the barotropic pressure. It can for most purposes be regarded as a
!  black box, and should not be changed without due care.

! Input arrays: Too many to document.
! Variable changed on output: f

      real f(imax,jmax),g1(2*jmax-1),g2(2*jmax-1),s(imax,jmax)
      real g1u(imax-1,jmax),g2u(imax-1,jmax)
      real g1v(imax,jmax-1),g2v(imax,jmax-1)
      real scu(jmax),scv(jmax),csv(jmax)
      data icall/0/
      save icall
      icall=icall+1
      ib=0
      ibt=0
      anormf=0.
      do j=2,jmax-1
       do i=2,imax-1
         anormf=anormf+abs(s(i,j))
       enddo
      enddo
      anormf=max(anormf,1.e-10)
  ! Start iterating
       do 100 n=1,maxits  
  	call bound1(1,f,g1,g2,scu,scv,rdx,rdy)
       anorm=0.
       do 40 j=2,jmax-1
        do 30 i=2,imax-1

!  Divergence template must be consistent with velocity templates 

        u1=-0.25*g1u(i,j)*rdy*( f(i+1,j+1)+f(i,j+1)
     .                         -f(i+1,j-1)-f(i,j-1) )
     .          -g2u(i,j)*scu(j)*rdx*(f(i+1,j)-f(i,j))
        u0=-0.25*g1u(i-1,j)*rdy*( f(i,j+1)+f(i-1,j+1)
     .                           -f(i,j-1)-f(i-1,j-1) )
     .          -g2u(i-1,j)*scu(j)*rdx*(f(i,j)-f(i-1,j))
        v1=0.25*g1v(i,j)*scv(j)*rdx*( f(i+1,j+1)+f(i+1,j)
     .                               -f(i-1,j+1)-f(i-1,j) )
     .         -g2v(i,j)*rdy*(f(i,j+1)-f(i,j))
        v0=0.25*g1v(i,j-1)*scv(j-1)*rdx*( f(i+1,j)+f(i+1,j-1)
     .                               -f(i-1,j)-f(i-1,j-1) )
     .         -g2v(i,j-1)*rdy*(f(i,j)-f(i,j-1))

! (gg is coeff. of f(i,j) )

       gg=(g2u(i,j)+g2u(i-1,j))*scu(j)**2*rdx**2
     .    +(g2v(i,j)*csv(j)+g2v(i,j-1)*csv(j-1))*scu(j)*rdy**2

! Calculate horizontal divergence
        div=scu(j)*( (u1-u0)*rdx + (v1*csv(j)-v0*csv(j-1))*rdy )

! Calculate residual
        resid=div-s(i,j)


!  Divergence template must be consistent with velocity templates 
! (gg is coeff. of f(i,j) )
! 	gg=(g2u(i,j)+g2u(i-1,j))*scu(j)**2*rdx**2
!     .    +(g2v(i,j)*csv(j)+g2v(i,j-1)*csv(j-1))*scu(j)*rdy**2
!
!        resid=
!     .scu(j)*( (
!     .-0.25*g1u(i,  j)*rdy*(f(i+1,j+1)+f(i,j+1)-f(i+1,j-1)-f(i,j-1))
!     .-g2u(i  ,j)*scu(j)*rdx*(f(i+1,j)-f(i,j))
!     .+0.25*g1u(i-1,j)*rdy*(f(i,j+1)+f(i-1,j+1)-f(i,j-1)-f(i-1,j-1))
!     .+g2u(i-1,j)*scu(j)*rdx*(f(i,j)-f(i-1,j)) )*rdx
!     .+(  (0.25*g1v(i,  j)*scv(j)*rdx
!     .     *(f(i+1,j+1)+f(i+1,j)-f(i-1,j+1)-f(i-1,j))
!     .  -g2v(i,  j)*rdy*(f(i,j+1)-f(i,j)))*csv(j)
!     . +(-0.25*g1v(i,j-1)*scv(j-1)*rdx
!     .     *(f(i+1,j)+f(i+1,j-1)-f(i-1,j)-f(i-1,j-1))
!     .  +g2v(i,j-1)*rdy*(f(i,j)-f(i,j-1)))*csv(j-1) )*rdy )
!     .   -s(i,j)
!

             anorm=anorm+abs(resid)
             f(i,j)=f(i,j)-omega1*resid/gg
30       continue
40      continue

! Following prints out the residual in various cases. 
! Note that if the maximum number of iterations
! are exceeded, the program does continue to run. 

       if((n.gt.1).and.(anorm.lt.tol1*anormf))then
         if(ipr.gt.0) then   ! only print if in verbose mode
	  resd=anorm/anormf
          print *,'No. of iterations', n,'   Residual = ',resd
          print *,'residual = ',resd,' Omega = ',omega1
         endif
	return
       endif
       if(ipr.gt.0 .and. mod(n,1000).eq.0)then
	print *, 'No. of iterations so far: ', n,ibt
	resd=anorm/anormf
        print *,'residual = ',resd,' Omega = ',omega1
       endif
100   continue
      print *, 'maxits exceeded'
	resd=anorm/anormf
 	print *,'residual = ',resd,' Omega = ',omega1
        print *, 'No. of iterations so far: ', n,ibt
      return
      end
